package com.xydl.cac.service.impl;

import com.xydl.cac.entity.IedDlConfig;
import com.xydl.cac.exception.BusinessException;
import com.xydl.cac.repository.IedDlConfigRepository;
import com.xydl.cac.service.IedDlConfigService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class IedDlConfigServiceImpl implements IedDlConfigService {

    @Resource
    IedDlConfigRepository repository;

    @Override
    public List<IedDlConfig> listAll() {
        List<IedDlConfig> list = repository.findAll();
        return list;
    }

    @Override
    public IedDlConfig get(Integer id) throws BusinessException {
        Optional<IedDlConfig> optional = repository.findById(id);
        if (!optional.isPresent()) {
            throw new BusinessException("未找到该记录");
        }
        return optional.get();
    }

    @Override
    public IedDlConfig add(IedDlConfig item) throws Exception {
        List<IedDlConfig> list = repository.findByDevId(item.getDevId());
        if (!CollectionUtils.isEmpty(list)) {
            throw new BusinessException("已存在该devId的配置");
        }
        item.setId(null);
        return repository.save(item);
    }

    @Override
    public void update(IedDlConfig item) throws Exception {
        repository.save(item);
    }

    @Override
    public void delete(Integer id) {
        repository.deleteById(id);
    }
}
